# encoding: utf-8

# The instantTranslate addon was used as a template for this code.
# It has been heavily modified to make sense in the new addon, but much of the great work of the original instantTranslate addon remains.
# code from Tyler Spivey's ClipCopy addon was also used for translating the last spoken text.
# Modified by Ian Reed

#__init__.py
# Copyright (C) 2012-2014 Aleksey Sadovoy AKA Lex <lex@progger.ru>,
#ruslan <ru2020slan@yandex.ru>,
#beqa <beqaprogger@gmail.com>
#This addon was been repacked and optimized for executing without standalone Python by Outsider <outsidepro@rambler.ru>.
#other nvda contributors
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.

from logHandler import log
import gui
import wx
import api
import textInfos
import tones
import scriptHandler
import globalPluginHandler
from functools import wraps
import queueHandler
import ui
import config
from locale import getdefaultlocale
import globalVars
from .interface import *
from .login import *
from .translator import Translator
from . import jgt_shared
from tones import beep
from time import sleep
import threading
import addonHandler
from .langslist import g
import speech
import six
import traceback

confspec = {
"from": "string(default=ja)",
"into": "string(default=en)",
"copytranslatedtext": "boolean(default=true)",
"username": 'string(default="")',
}
config.conf.spec["JGT"] = confspec

_addonDir = os.path.join(os.path.dirname(__file__), "..", "..")
if isinstance(_addonDir, bytes):
	_addonDir = _addonDir.decode("mbcs")
_curAddon = addonHandler.Addon(_addonDir)
_addonSummary = _curAddon.manifest['summary']
addonHandler.initTranslation()

def finally_(func, final):
	"""Calls final after func, even if it fails."""
	def wrap(f):
		@wraps(f)
		def new(*args, **kwargs):
			try:
				func(*args, **kwargs)
			finally:
				final()
		return new
	return wrap(final)

def getClipboardValue():
	try:
		text = api.getClipData()
	except:
		text = ""
	if not text or not isinstance(text,str) or text.isspace():
		text = "" # return a consistent no text value.
	return text

def replaceNumbers(input):
	origNumbers = []
	length = len(input)
	output = ""
	inNumber = False
	startIndex = -1
	nextNumberIndex = 0
	for i in range(0, length):
		c = input[i]
		if c.isdigit():
			if not inNumber:
				startIndex = i
				output += str(nextNumberIndex)
				nextNumberIndex += 1
				inNumber = True
		else: # not in a number
			output += c
			if inNumber:
				origNumbers.append(input[startIndex:i])
				inNumber = False
	# special case: get the last number when the input text ends with a number
	if inNumber:
		origNumbers.append(input[startIndex:length])
	return (output, origNumbers)

def undoReplaceNumbers(input, origNumbers):
	if len(origNumbers) == 0:
		return input # no numbers need to be replaced
	output = ""
	inNumber = False
	nextNumberIndex = 0
	for i in range(0, len(input)):
		c = input[i]
		if c.isdigit():
			if not inNumber:
				if len(origNumbers) > nextNumberIndex:
					output += origNumbers[nextNumberIndex]
					nextNumberIndex += 1
					inNumber = True
		else: # not in a number
			output += c
			inNumber = False
	return output

class GlobalPlugin(globalPluginHandler.GlobalPlugin):
	scriptCategory = six.text_type(_addonSummary)

	def __init__(self, *args, **kwargs):
		super(GlobalPlugin, self).__init__(*args, **kwargs)
		if globalVars.appArgs.secure:
			return
		self.createMenu()
		self.getUpdatedGlobalVars()
		self.toggling = False
		self.maxCachedResults = 200
		self.cachedResults = []
		self.sessionID = None
		self.lastThread = None
		# setup the speech proxy
		jgt_shared.JGT_globalPlugin = self
		jgt_shared.JGT_oldSpeak = speech.speak
		speech.speak = jgt_shared.JGT_speak

	lastClipboardValue = ""
	def clipboardChecker(self):
		# this method is called on its own thread.
		inGame = False
		while jgt_shared.JGT_autoClipboard:
			if not jgt_shared.getInGame():
				sleep(1) # we don't check if we are in the right app as often if we weren't in the right app recently.
				inGame = False
			else:
				if not inGame: # this helps us avoid auto translating whatever was in the clipboard when they first switch into the game.
					self.lastClipboardValue = getClipboardValue()
				inGame = True
				sleep(0.02)
				text = getClipboardValue()
				if text != self.lastClipboardValue:
					self.lastClipboardValue = text
					(shouldTranslate, shouldSpeak) = jgt_shared.getShouldTranslate(text)
					if shouldTranslate:
						try:
							self.startTranslateThread([text])
						except Exception as e:
							jgt_shared.queueMessage("caught exception in clipboardChecker")
							jgt_shared.queueMessage(str(e))
							tb = tb = traceback.format_exc()
							jgt_shared.queueMessage(tb)
							pass
					if shouldSpeak:
						jgt_shared.queueMessage(text)

	def getUpdatedGlobalVars(self):
		global lang_from, lang_to, copyTranslation, username
		# source language
		lang_from = config.conf['JGT']['from']
		# target language
		lang_to = config.conf['JGT']['into']
		# determine whether to copy translation on clipboard
		copyTranslation = config.conf['JGT']['copytranslatedtext']
		# username
		username = config.conf['JGT']['username']

	def createMenu(self):
		self.prefsMenu = gui.mainFrame.sysTrayIcon.menu.GetMenuItems()[0].GetSubMenu()
		self.JGTSettingsItem = self.prefsMenu.Append(wx.ID_ANY,
			# Translators: name of the option in the menu.
			_("&JGT Settings..."),
			# Translators: tooltip text for the menu item.
			_("Select languages to be used for translation."))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU , lambda e : gui.mainFrame._popupSettingsDialog(JGTSettingsDialog), self.JGTSettingsItem)
		# handle the login menu, I really wanted to pop this up on the fly when a session had expired, but I couldn't get that working
		self.JGTLoginItem = self.prefsMenu.Append(wx.ID_ANY,
			# Translators: name of the option in the menu.
			_("&JGT Login..."),
			# Translators: tooltip text for the menu item.
			_("Login to JGT service."))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU , lambda e : gui.mainFrame._popupSettingsDialog(JGTLoginDialog, self), self.JGTLoginItem)

	def terminate(self):
		speech.speak = jgt_shared.JGT_oldSpeak
		try:
			self.prefsMenu.RemoveItem(self.JGTSettingsItem)
		except wx.PyDeadObjectError:
			pass

	def script_toggleAutoLastSpoken(self, gesture):
		jgt_shared.JGT_autoLastSpoken = not jgt_shared.JGT_autoLastSpoken
		if jgt_shared.JGT_autoLastSpoken:
			jgt_shared.queueMessage("enabled auto translate last spoken")
			#if jgt_shared.JGT_autoClipboard:
			#	jgt_shared.JGT_autoClipboard = False
			#	jgt_shared.queueMessage("disabled auto translate clipboard, since both can not be on at the same time")
		else:
			jgt_shared.queueMessage("disabled auto translate last spoken")
	script_toggleAutoLastSpoken.__doc__=_("Toggle auto translate last spoken")

	def script_toggleAutoClipboard(self, gesture):
		jgt_shared.JGT_autoClipboard = not jgt_shared.JGT_autoClipboard
		if jgt_shared.JGT_autoClipboard:
			jgt_shared.queueMessage("enabled auto translate clipboard")
			#if jgt_shared.JGT_autoLastSpoken:
			#	jgt_shared.JGT_autoLastSpoken = False
			#	jgt_shared.queueMessage("disabled auto translate last spoken, since both can not be on at the same time")
			threading.Thread(target=self.clipboardChecker).start()
		else:
			jgt_shared.queueMessage("disabled auto translate clipboard")
	script_toggleAutoClipboard.__doc__=_("Toggle auto translate clipboard")

	def script_translateClipboard(self, gesture):
		text = getClipboardValue()
		if text == "":
			# Translators: message presented when user presses the shortcut key for translating clipboard text but the clipboard is empty.
			jgt_shared.queueMessage(_("There is no text on the clipboard"))
		else:
			(shouldTranslate, shouldSpeak) = jgt_shared.getShouldTranslate(text)
			if shouldTranslate:
				self.startTranslateThread([text])
			if shouldSpeak:
				jgt_shared.queueMessage(text)
	# Translators: message presented in input help mode, when user presses the shortcut keys for this addon.
	script_translateClipboard.__doc__=_("Translate clipboard")

	def script_translateLastSpoken(self, gesture):
		#jgt_shared.ianDebug()
		text = jgt_shared.JGT_data
		if not text or not isinstance(text,str) or text.isspace():
			jgt_shared.queueMessage(_("The last spoken text was empty."))
		else:
			(shouldTranslate, shouldSpeak) = jgt_shared.getShouldTranslate(text)
			if shouldTranslate:
				self.startTranslateThread([text])
			if shouldSpeak:
				jgt_shared.queueMessage(text)
	# Translators: message presented in input help mode, when user presses the shortcut keys for this addon.
	script_translateLastSpoken.__doc__=_("Translate last spoken")

	def startTranslateThread(self, speechSequence):
		# by having everyone call this instead of the translate method directly, we can stop more than one thread from translating the same text at the same time.
		# we also check the cache before starting a thread to query the web service.
		text = speechSequence[0]
		# had to move the call to ChangeUnicodeNumbersAndOtherSymbolsToAscii here so it occurs before we replace numbers. Otherwise the 0 to 1`1000 ranges in ShadowRine settings remain as Japanese numbers.
		text = jgt_shared.ChangeUnicodeNumbersAndOtherSymbolsToAscii(text)
		(text, origNumbers) = replaceNumbers(text)
		(cachedVal, index) = self.findResultInCache(text, lang_to, lang_from)
		if cachedVal != None:
			(_, _, _, translation) = cachedVal
			translation = undoReplaceNumbers(translation, origNumbers)
			# there's something wrong with removing from the cache, so I'll just have to stop doing this behavior that is intended to move the item to the top of the cache.
			#self.addResultToCache(text, translation, removeIndex=index)
			#jgt_shared.queueMessage("from cache " + translation)
			speechSequence[0] = translation
			jgt_shared.queueSequence(speechSequence)
			self.copyResult(translation)
			return
		# check the large cache that is stored on disk.
		if username != "":
			translation = jgt_shared.getFromFileCache(text, lang_to)
			if translation != None:
				translation = self.undoNumbersCacheAndSpeak(text, translation, speechSequence, origNumbers)
				jgt_shared.logToOutputFile(translation)
				return
		if self.lastThread != None and self.lastThread.isAlive():
			#jgt_shared.queueMessage("skipped")
			return # we don't want to have more than one translation thread running at a time.
		self.lastThread = threading.Thread(target=self.translate, args=(speechSequence,origNumbers,))
		self.lastThread.start()

	def translate(self, speechSequence, origNumbers):
		if self.sessionID == None:
			# I never could get a dialog to appear, if it was appearing it wasn't in the alt tab list and did not have focus.
			jgt_shared.queueMessage("You must first login using the JGT login dialog in preferences")
			# need to display a login dialog
			#gui.mainFrame.Show()
			#gui.mainFrame.Hide()
			#gui.mainFrame.showGui()
			#gui.mainFrame._popupSettingsDialog(JGTSettingsDialog)
			#gui.mainFrame._popupSettingsDialog(JGTLoginDialog, self)
			#jgt_shared.queueMessage("got here 2")
			return
		self.getUpdatedGlobalVars()
		translation = None
		#jgt_shared.queueMessage(str(len(self.cachedResults)))
		title = jgt_shared.getTitle()
		text = speechSequence[0]
		myTranslator = Translator(lang_from, lang_to, text, self.sessionID, title)
		myTranslator.start()
		i=0
		while myTranslator.isAlive():
			sleep(0.05)
			i+=1
			if i == 20:
				beep(500, 100)
				i = 0
		myTranslator.join()
		translation = myTranslator.translation
		translation = self.undoNumbersCacheAndSpeak(text, translation, speechSequence, origNumbers)
		return translation

	def undoNumbersCacheAndSpeak(self, original, translation, speechSequence, origNumbers):
		translation = undoReplaceNumbers(translation, origNumbers)
		if translation != '':
			self.addResultToCache(original, translation)
		speechSequence[0] = translation
		jgt_shared.queueSequence(speechSequence)
		self.copyResult(translation)
		return translation

	def addResultToCache(self, text, translation, removeIndex=-1):
		if removeIndex != -1:
			self.cachedResults.pop(removeIndex)
		elif len(self.cachedResults) == self.maxCachedResults:
			self.cachedResults.pop(0)
		self.cachedResults.append((text, lang_to, lang_from, translation))

	def findResultInCache(self, text, lang_to, lang_from):
		# returns the (text, lang_to, lang_from, translation tuple and the index it was found at.
		# so (val, index)
		for i in reversed(range(0, len(self.cachedResults))):
			val = self.cachedResults[i]
			(iText, iLangTo, iLangFrom, iTranslation) = val
			if (iText == text or iTranslation == text) and lang_from == iLangFrom and lang_to == iLangTo:
				#jgt_shared.queueMessage("found at " + str(i) + " of " + str(len(self.cachedResults)))
				return (val, i)
		return (None, -1)

	def copyResult(self, translation, ignoreSetting=False):
		global copyTranslation
		self.getUpdatedGlobalVars()
		if ignoreSetting or copyTranslation:
			try:
				# sometimes we get exceptions when saving the clipboard value, but oddly it still seems to save, so we want to update lastClipboardValue before hand.
				self.lastClipboardValue = translation
				result = api.copyToClip(translation)
			except Exception as e:
				#jgt_shared.queueMessage(str(e))
				pass
			# we populate the lastClipboardValue after updating the clipboard, because we don't need the auto translate clipboard feature to translate the stuff we just translated.
			#self.lastClipboardValue = getClipboardValue()
			# it looks like this failed when I was using the entire json string as the result, but now that I just use the actual translation text it is fine.
			# perhaps the special characters needed to be escaped in some way, oh well, it works well enough now.
			#if not result:
			#	jgt_shared.queueMessage("failed to copy to clipboard")

	__gestures = {
		#"kb:NVDA+shift+h": "translateClipboard", # not very functional because it only makes sense when the Japanese keyboard is enabled, and the caps lock+shift key combo changes some language settings when the Japanese keyboard is enabled.
		"kb:NVDA+shift+control+h": "toggleAutoClipboard",
		"kb:NVDA+shift+y": "translateLastSpoken",
		"kb:NVDA+shift+control+y": "toggleAutoLastSpoken",
	}
