#interface.py
# Copyright (C) 2012-2014 Aleksey Sadovoy AKA Lex <lex@progger.ru>,
#ruslan <ru2020slan@yandex.ru>,
#beqa <beqaprogger@gmail.com>
#other nvda contributors
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.

import os.path
import wx
import gui
from .langslist import langslist
import globalVars
import config
import addonHandler
from copy import deepcopy
from . import jgt_shared

addonHandler.initTranslation()

class JGTSettingsDialog(gui.SettingsDialog):
	# Translators: name of the dialog.
	title = _("Japanese Games Translator Settings")

	def __init__(self, parent):
		super(JGTSettingsDialog, self).__init__(parent)

	def makeSettings(self, sizer):
		# Translators: Help message for a dialog.
		helpLabel = wx.StaticText(self, label=_("Select translation source and target language:"))
		helpLabel.Wrap(self.GetSize()[0])
		sizer.Add(helpLabel)
		fromSizer = wx.BoxSizer(wx.HORIZONTAL)
		# Translators: A setting in addon settings dialog.
		fromLabel = wx.StaticText(self, label=_("Source language:"))
		fromSizer.Add(fromLabel)
		self._fromChoice = wx.Choice(self, choices=["ja"])
		fromSizer.Add(self._fromChoice)
		intoSizer = wx.BoxSizer(wx.HORIZONTAL)
		# Translators: A setting in addon settings dialog.
		intoLabel = wx.StaticText(self, label=_("Target language:"))
		intoSizer.Add(intoLabel)
		self._intoChoice = wx.Choice(self, choices=["en","es","fr","de","pt","ru","pl","ko","tr"])
		intoSizer.Add(self._intoChoice)
		sizer.Add(fromSizer)
		sizer.Add(intoSizer)
		# Translators: A setting in addon settings dialog.
		self.copyTranslationChk = wx.CheckBox(self, label=_("Copy translation result to clipboard"))
		self.copyTranslationChk.SetValue(config.conf['JGT']['copytranslatedtext'])
		sizer.Add(self.copyTranslationChk)
		iLang_from = self._fromChoice.FindString(config.conf['JGT']['from'])
		iLang_to = self._intoChoice.FindString(config.conf['JGT']['into'])
		self._fromChoice.Select(iLang_from)
		self._intoChoice.Select(iLang_to)

	def postInit(self):
		self._fromChoice.SetFocus()

	def onOk(self, event):
		# Update Configuration
		config.conf['JGT']['from'] = self._fromChoice.GetStringSelection()
		toLang = self._intoChoice.GetStringSelection()
		config.conf['JGT']['into'] = toLang
		config.conf['JGT']['copytranslatedtext'] = self.copyTranslationChk.GetValue()
		jgt_shared.loadFileCache(toLang)
		super(JGTSettingsDialog, self).onOk(event)
