import ui
from logHandler import log
import os.path
import wx
import gui
from .langslist import langslist
import globalVars
import config
import addonHandler
from copy import deepcopy
import json
import sys

if sys.version_info.major < 3:
	impPath = os.path.abspath(os.path.dirname(__file__))
	sys.path.append(impPath)
	from . import urllib2 as urllibRequest
	from .urllib2 import quote
	del sys.path[-1]
else:
	import urllib.request as urllibRequest
	from urllib.parse import quote

class JGTLoginDialog(gui.SettingsDialog):
	# Translators: name of the dialog.
	title = _("Login to the JGT service")

	def __init__(self, parent, sessionHolder):
		super(JGTLoginDialog, self).__init__(parent)
		self.sessionHolder = sessionHolder

	def makeSettings(self, sizer):
		# Translators: Help message for a dialog.
		helpLabel = wx.StaticText(self, label=_("Enter your username and password for JGT:"))
		helpLabel.Wrap(self.GetSize()[0])
		sizer.Add(helpLabel)
		userSizer = wx.BoxSizer(wx.HORIZONTAL)
		# Translators: A setting in addon settings dialog.
		userLabel = wx.StaticText(self, label=_("Username:"))
		userSizer.Add(userLabel)
		self._userText = wx.TextCtrl(self)
		self._userText.SetValue(config.conf['JGT']['username'])
		userSizer.Add(self._userText)
		passSizer = wx.BoxSizer(wx.HORIZONTAL)
		# Translators: A setting in addon settings dialog.
		passLabel = wx.StaticText(self, label=_("Password:"))
		passSizer.Add(passLabel)
		self._passText = wx.TextCtrl(self, style=wx.TE_PASSWORD)
		passSizer.Add(self._passText)
		sizer.Add(userSizer)
		sizer.Add(passSizer)

	def postInit(self):
		self._userText.SetFocus()

	def onOk(self, event):
		# attempt to login
		user = self._userText.GetValue()
		config.conf['JGT']['username'] = user
		# pass is a python keyword so we had to rename this variable
		password = self._passText.GetValue()
		# build the request url
		urlTemplate = 'https://BlindGamers.com/api/login1?u={user}&p={password}&v={version}'
		# if having a problem with the SSL certificate you can use this url instead
		#urlTemplate = 'http://BlindAudioGames.com/api/login1?u={user}&p={password}&v={version}'
		#urlTemplate = 'http://localhost:55255/api/login1?u={user}&p={password}&v={version}'
		user = quote(user.encode('utf-8'))
		password = quote(password.encode('utf-8'))
		version = quote("JGT 2.0")
		url = urlTemplate.format(user=user, password=password, version=version)
		try:
			data = json.load(urllibRequest.urlopen(url))
			error = data[0]
			if error != "":
				gui.messageBox(error,_("Error"),style=wx.OK | wx.ICON_ERROR)
			self.sessionHolder.sessionID = data[1]
			ourVersion = "2.1"
			latestJGTVersion = data[2]
			if latestJGTVersion != ourVersion:
				gui.messageBox("You are not using the latest JGT version, you should consider downloading a new version from BlindGamers.com","",style=wx.OK | wx.ICON_ERROR)
		except Exception as e:
			# get the message from the exception
			msg = str(e)
			gui.messageBox(msg, _("Error"),style=wx.OK | wx.ICON_ERROR)
		# needed to run the super class version last, because NVDA 2018.2 disposes the _userText control before I can get the value from it.
		super(JGTLoginDialog, self).onOk(event)
